unit Main_U;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Spin, ExtCtrls, StdCtrls;

type
  TMain_F = class(TForm)
    OpenDialog1: TOpenDialog;
    edVolFilename: TEdit;
    pbBrowse1: TButton;
    Label1: TLabel;
    rbRandomOptions: TGroupBox;
    rbPseudoRandomSecond: TRadioButton;
    rbRandomFromFile: TRadioButton;
    edRandomFilename: TEdit;
    Label2: TLabel;
    pbBrowse2: TButton;
    pbModify: TButton;
    pbClose: TButton;
    Label3: TLabel;
    lblVersion: TLabel;
    rbSwapKeyBlocks: TRadioButton;
    procedure pbBrowse1Click(Sender: TObject);
    procedure pbBrowse2Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure pbCloseClick(Sender: TObject);
    procedure pbModifyClick(Sender: TObject);
  private
    procedure OperateOnFile();
  public
    { Public declarations }
  end;

var
  Main_F: TMain_F;

implementation

{$R *.DFM}

uses
  SDUGeneral,
  SDUFileIterator_U;

const
    // From the ScramDisk source:
    //   if (WAVformat==0)       // BACKUP COPY OF CRITCIAL DATA, BUT NOT ON WAV....(too obvious)
    //   memcpy (o,i,1024+32);   //copy scrambled keytable to end of table. In case of disk error...
    //                           //can be recovered...... INCREASE +32 if max key size is increased
    // Therefore...
    KEYBLOCK_SIZE = 1024+32;

    KEYBLOCK1_OFFSET = 0;
    KEYBLOCK2_OFFSET = 6144;

procedure TMain_F.pbBrowse1Click(Sender: TObject);
begin
  opendialog1.filterindex := 1;
  SDUOpenSaveDialogSetup(opendialog1, edVolFilename.text);
  if opendialog1.execute() then
    begin
    edVolFilename.text := opendialog1.filename;
    end;

end;

procedure TMain_F.pbBrowse2Click(Sender: TObject);
begin
  opendialog1.filterindex := 2;
  SDUOpenSaveDialogSetup(opendialog1, edRandomFilename.text);
  if opendialog1.execute() then
    begin
    edRandomFilename.text := opendialog1.filename;
    end;

end;

procedure TMain_F.FormCreate(Sender: TObject);
var
  majorVersion, minorVersion, releaseVersion, buildVersion: integer;
begin
  SDUGetVersionInfo('', majorVersion, minorVersion, releaseVersion, buildVersion);
  lblVersion.caption := Format('v%d.%.2d.%.2d.%.4d', [majorVersion, minorVersion, releaseVersion, buildVersion]);

end;

procedure TMain_F.pbCloseClick(Sender: TObject);
begin
  Close;

end;

procedure TMain_F.pbModifyClick(Sender: TObject);
begin
  if not(FileExists(edVolFilename.text)) then
    begin
    showmessage('Volume file "'+edVolFilename.text+'" does not exist');
    exit;
    end;

  OperateOnFile();

  showmessage('Finished.');

end;


procedure TMain_F.OperateOnFile();
var
  buffer1: array [0..(KEYBLOCK_SIZE-1)] of byte;
  buffer2: array [0..(KEYBLOCK_SIZE-1)] of byte;
  tmpByte: byte;
  fileHandle: THandle;
  dataFileSizeLo: DWORD;
  dataFileSizeHi: DWORD;
  bytesWritten: DWORD;
  i: integer;
begin
  // Read in both of the existing key blocks
  fileHandle := CreateFile(PChar(edVolFilename.text),
                           GENERIC_READ,
                           0,
                           nil,
                           OPEN_EXISTING,
                           FILE_ATTRIBUTE_NORMAL,
                           0);

  if fileHandle=INVALID_HANDLE_VALUE then
    begin
    showmessage('Unable to open volume file:'+#13+#10+edRandomFilename.text);
    exit;
    end;

  dataFileSizeLo := GetFileSize(fileHandle, @dataFileSizeHi);
  if (dataFileSizeLo<DWORD(KEYBLOCK_SIZE+KEYBLOCK2_OFFSET)) AND (dataFileSizeHi<1) then
    begin
    showmessage('This is not a ScramDisk volume - it''s WAY too small!');
    CloseHandle(fileHandle);
    exit;
    end;

  SetFilePointer(fileHandle, KEYBLOCK1_OFFSET, nil, FILE_BEGIN);
  ReadFile(fileHandle, buffer1, KEYBLOCK_SIZE, bytesWritten, nil);

  SetFilePointer(fileHandle, KEYBLOCK2_OFFSET, nil, FILE_BEGIN);
  ReadFile(fileHandle, buffer2, KEYBLOCK_SIZE, bytesWritten, nil);

  CloseHandle(fileHandle);



  // Change the key blocks as appropriate
  if rbSwapKeyBlocks.checked then
    begin
    for i:=low(buffer2) to high(buffer2) do
      begin
      tmpByte:=buffer1[i];
      buffer1[i] := buffer2[i];
      buffer2[i] := tmpByte;
      end;
    end
  else if rbPseudoRandomSecond.checked then
    begin
    Randomize;
    for i:=low(buffer2) to high(buffer2) do
      begin
      buffer2[i] := random(255);
      end;
    end
  else if rbRandomFromFile.checked then
    begin
    // Check that the random data file (if needed) contains enough data
    fileHandle := CreateFile(PChar(edRandomFilename.text),
                             GENERIC_READ,
                             0,
                             nil,
                             OPEN_EXISTING,
                             FILE_ATTRIBUTE_NORMAL,
                             0);

    if fileHandle=INVALID_HANDLE_VALUE then
      begin
      showmessage('Unable to open file containing random data:'+#13+#10+edRandomFilename.text);
      exit;
      end;

    dataFileSizeLo := GetFileSize(fileHandle, @dataFileSizeHi);
    if (dataFileSizeLo<DWORD(KEYBLOCK_SIZE)) AND (dataFileSizeHi<1) then
      begin
      showmessage('File containing random data too small. Must be at least '+inttostr(KEYBLOCK_SIZE)+' bytes');
      CloseHandle(fileHandle);
      exit;
      end;

    SetFilePointer(fileHandle, 0, nil, FILE_BEGIN);

    ReadFile(fileHandle, buffer2, KEYBLOCK_SIZE, bytesWritten, nil);
    CloseHandle(fileHandle);
    end;


    
  // Write out the keyblocks
  fileHandle := CreateFile(PChar(edVolFilename.text),
                           GENERIC_WRITE,
                           0,
                           nil,
                           OPEN_EXISTING,
                           FILE_ATTRIBUTE_NORMAL,
                           0);

  if fileHandle=INVALID_HANDLE_VALUE then
    begin
    showmessage('Unable to open volume file:'+#13+#10+edRandomFilename.text);
    exit;
    end;

  SetFilePointer(fileHandle, KEYBLOCK1_OFFSET, nil, FILE_BEGIN);
  WriteFile(fileHandle, buffer1, KEYBLOCK_SIZE, bytesWritten, nil);

  SetFilePointer(fileHandle, KEYBLOCK2_OFFSET, nil, FILE_BEGIN);
  WriteFile(fileHandle, buffer2, KEYBLOCK_SIZE, bytesWritten, nil);

  CloseHandle(fileHandle);


end;

END.

